/*
  "Timer2TaskScheduler"
  V1.0 / Copyright (C) 2017, T.Rode (DL1DUZ)

  Simple Task-scheduler library for Arduino using Timer 2
 (conflicting with the tone() and analogWrite() at pins 3 & 11).

  Permission is granted to use, copy, modify, and distribute this software
  and documentation for non-commercial purposes.

*/

#ifndef Timer2TaskScheduler_h_
#define Timer2TaskScheduler_h_

#if (ARDUINO >= 100)
#include <Arduino.h> 
#else
#include <WProgram.h> 
#endif

#include <stdint.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/atomic.h>

//################################################################################################################
//definitions
//################################################################################################################

class SchedulerClass {
	
  private:
  
    typedef void (*f_ptr)();

    uint8_t is_initialized = 0;
    uint8_t elements = 0;
	
    struct task {
      uint8_t is_scheduled = 0;
      volatile uint32_t duetime;
      f_ptr f_pointer;
    };

    task* task_array_pointer;
	task* dummy;
    uint8_t next_free_element = 0;

    uint32_t processTask(f_ptr function, uint8_t id, uint32_t time);
	
  public:

    boolean init(byte size, byte cycle);
    byte getTotalSize();
	byte getUsedSize();
    byte getFreeSize();
    void runIfDue(byte task);
    unsigned long getDueTime(f_ptr function);
    boolean scheduleTask(f_ptr function, unsigned long time);
    boolean unscheduleTask(f_ptr function);
    boolean isScheduled(f_ptr function);

    static SchedulerClass *this_instance;
	void isr();

};

extern SchedulerClass SDL;

//################################################################################################################

#endif // Timer2TaskScheduler_h_

